
//--------------------------------------<                        axlib v1.1                             >-------------------------------------------
//--------------------------------------<       16x2 >-------------------------------------------
//--------------------------------------<            2015 http://www.avrki.ru            >-------------------------------------------


#ifndef LCD_H_
#define LCD_H_

#if !defined(MAIN_INIT_H_)
#error "You must included (#include \"main_init.h\") befor use (#include <axlib/lcd.h>)."
#endif

#include <avr/io.h>
#include <util/delay.h>

#if defined(R74HC595_LCD)
	#include <axlib/74hc595.h>
#endif

#include <axlib/type_var.h>

#if RUS		//        
	#include <axlib/font.h>
#endif



//-------------------------------------------------------------------------
//	 
//-------------------------------------------------------------------------

//     

#if !defined(R74HC595_LCD)
	#define E_SET()		(PORT_E |= (1 << E))		//   
	#define E_RESET()	(PORT_E &= ~(1 << E))		//   
	#define RS_COM()	(PORT_RS &= ~(1 << RS))		//  
	#define RS_DATA()	(PORT_RS |= (1 << RS))		//  

//       74HC595
#else
	#define E_SET(data)		(data |= (1 << E))		//   
	#define E_RESET(data)	(data &= ~(1 << E))		//   
	#define RS_COM(data)	(data &= ~(1 << RS))	//  
	#define RS_DATA(data)	(data |= (1 << RS))		//  
#endif

//-------------------------------------------------------------------------
//	      
//	 :
//			BYTE data
//			  
//-------------------------------------------------------------------------

void lcd_out(BYTE data)
{
//     

#if !defined(R74HC595_LCD)

	if(data & 0x01)
	{
		PORT_D4 |= (1 << D4);
	}
	else
	{
		PORT_D4 &= ~(1 << D4);
	}
	if(data & 0x02)
	{
		PORT_D5 |= (1 << D5);
	}
	else
	{
		PORT_D5 &= ~(1 << D5);
	}
	if(data & 0x04)
	{
		PORT_D6 |= (1 << D6);
	}
	else
	{
		PORT_D6 &= ~(1 << D6);
	}
	if(data & 0x08)
	{
		PORT_D7 |= (1 << D7);
	}
	else
	{
		PORT_D7 &= ~(1 << D7);
	}
	
//       74HC595
#else
	
	reg_74hc595_byte(data);

#endif
}

//-------------------------------------------------------------------------
//	    
//	 :
//			BYTE com
//			
//-------------------------------------------------------------------------

void lcd_com(BYTE com)
{	
//     

#if !defined(R74HC595_LCD)

	RS_COM();				//  
	lcd_out(com);			//    
	E_SET();				//  
	E_RESET();				//  
	_delay_ms(4);			//   

//       74HC595
#else

	RS_COM(com);						//  
	reg_74hc595_byte(E_SET(com));		//  
	reg_74hc595_byte(E_RESET(com));		//  
	_delay_ms(4);						//   
	
#endif

}

//-------------------------------------------------------------------------
//	  
//	 :
//			BYTE lcd
//			0 -    ,   
//			1 -    ,      
//			2 -    ,    
//			3 -    ,       
//-------------------------------------------------------------------------
void lcd_init(BYTE lcd)
{
//     

#if !defined(R74HC595_LCD)

	DDR_RS |= (1 << RS);
	DDR_E  |= (1 << E);
	DDR_D4 |= (1 << D4);
	DDR_D5 |= (1 << D5);
	DDR_D6 |= (1 << D6);
	DDR_D7 |= (1 << D7);
	
	switch (lcd)
	{
		case 0: lcd = 0x0C; break;
		case 1: lcd = 0x0D; break;
		case 2: lcd = 0x0E; break;
		case 3: lcd = 0x0F; break;
	}
	
	_delay_ms(20);		//     20 
	
	lcd_com(0x03);		//   4-  
	_delay_us(40);
	lcd_com(0x03);		//   4-  
	_delay_us(40);
	lcd_com(0x03);		//   4-  
	_delay_us(40);
	lcd_com(0x02);		//   4-  
	_delay_us(40);
	lcd_com(0x02);		//  
	lcd_com(0x08);		//  
	lcd_com(0x00);		//  
	lcd_com(0x08);		//  
	lcd_com(0x00);		//  
	lcd_com(0x01);		//  
	lcd_com(0x00);		//    
	lcd_com(0x06);		//    
	lcd_com(0x00);		//     
	lcd_com(lcd);		//     

//       74HC595	
#else
	
	reg_74hc595_init();
	
	switch (lcd)
	{
		case 0: lcd = 0xC0; break;
		case 1: lcd = 0xD0; break;
		case 2: lcd = 0xE0; break;
		case 3: lcd = 0xF0; break;
	}
	
	_delay_ms(20);		//     20 
	
	lcd_com(0x30);		//   4-  
	_delay_us(40);
	lcd_com(0x30);		//   4-  
	_delay_us(40);
	lcd_com(0x30);		//   4-  
	_delay_us(40);
	lcd_com(0x20);		//   4-  
	_delay_us(40);
	lcd_com(0x20);		//  
	lcd_com(0x80);		//  
	lcd_com(0x00);		//  
	lcd_com(0x80);		//  
	lcd_com(0x00);		//  
	lcd_com(0x10);		//  
	lcd_com(0x00);		//    
	lcd_com(0x60);		//    
	lcd_com(0x00);		//     
	lcd_com(lcd);		//     
	
#endif
}

//-------------------------------------------------------------------------
//	    
//	 :
//			BYTE data
//			 
//-------------------------------------------------------------------------
void lcd_char_out(BYTE data)
{
//     
#if !defined(R74HC595_LCD)

	RS_DATA();							//  
	
	lcd_out(data >> 4);					//   4 
	E_SET();
	E_RESET();
	
	lcd_out(data & 0x0F);				//   4 
	E_SET();
	E_RESET();
	
	_delay_ms(4);						//   

//       74HC595
#else

	BYTE temp = 0;
	temp = (data & 0xF0);				//   4 
	RS_DATA(temp);						//  
	reg_74hc595_byte(E_SET(temp));		//  
	reg_74hc595_byte(E_RESET(temp));	//  
	
	temp = (data << 4);					//   4 
	RS_DATA(temp);						//  
	reg_74hc595_byte(E_SET(temp));		//  
	reg_74hc595_byte(E_RESET(temp));	//  
	
	_delay_ms(2);						//   
	
#endif

}

//-------------------------------------------------------------------------
//	  
//	 :
//			BYTE *str
//			     .
//-------------------------------------------------------------------------
void lcd_str_out(BYTE *str)
{
	while((*str) != '\0')
	{
		lcd_char_out(*str);
		str++;
	}
}

//-------------------------------------------------------------------------
//	     X, Y
//	 :
//			BYTE x, BYTE y
//			 -   0  39.  
//			y -   0  3.   
//-------------------------------------------------------------------------
void lcd_gotoxy(BYTE x, BYTE y)
{
	if(x > 39) x = 39;
	if(x < 0) x = 0;
	if(y > 3) y = 3;
	if(y < 0) y = 0;
	
	BYTE temp = 0x00;
	
//     

#if !defined(R74HC595_LCD)	
	RS_COM();
	
	switch (y)
	{
		case 0:
		{
			temp |= (0x80 + x);
			lcd_com(temp >> 4);			//   4 
			lcd_com(temp & 0x0F);		//   4 
			break;
		}
		case 1:
		{
			temp |= (0xC0 + x);
			lcd_com(temp >> 4);			//   4 
			lcd_com(temp & 0x0F);		//   4 
			break;
		}
		case 2:
		{
			temp |= (0x94 + x);
			lcd_com(temp >> 4);			//   4 
			lcd_com(temp & 0x0F);		//   4 
			break;
		}
		case 3:
		{
			temp |= (0xD4 + x);
			lcd_com(temp >> 4);			//   4 
			lcd_com(temp & 0x0F);		//   4 
			break;
		}
	}
#else
	RS_COM(temp);
	
	switch (y)
	{
		case 0:
		{
			temp |= (0x80 + x);
			lcd_com(temp & 0xF0);		//   4 
			lcd_com(temp >> 4);			//   4 
			break;
		}
		case 1:
		{
			temp |= (0xC0 + x);
			lcd_com(temp & 0xF0);		//   4 
			lcd_com(temp >> 4);			//   4 
			break;
		}
		case 2:
		{
			temp |= (0x94 + x);
			lcd_com(temp & 0xF0);		//   4 
			lcd_com(temp >> 4);			//   4 
			break;
		}
		case 3:
		{
			temp |= (0xD4 + x);
			lcd_com(temp & 0xF0);		//   4 
			lcd_com(temp >> 4);			//   4 
			break;
		}
	}
#endif
	
	
}

//-------------------------------------------------------------------------
//	       0 , 0 
//-------------------------------------------------------------------------
void lcd_clear(void)
{
	lcd_com(0x00);
	lcd_com(0x01);
}

//-------------------------------------------------------------------------
//	       
//	 :
//		BYTE simbol -   CGRAM  0  15
//		BYTE *str -       
//-------------------------------------------------------------------------
void lcd_simbol(BYTE simbol, BYTE *str) 
{
	simbol = (8 * simbol) | 0x40;
	
#if !defined(R74HC595_LCD)		
	
	lcd_com(simbol >> 4);
	lcd_com(simbol & 0x0F);
#else

	lcd_com(simbol & 0xF0);
	lcd_com(simbol >> 4);
#endif
	
	for(BYTE i = 0; i <= 7; i++)				
	{
		lcd_char_out(*str);
		str++;
	}
	
	lcd_gotoxy(0,0);

}


#if RUS	
//-------------------------------------------------------------------------
//	       
//	 :
//		BYTE simbol -   CGRAM  0  15
//		BYTE add -     lcd_font[add][x]
//
//-------------------------------------------------------------------------
void lcd_fsimbol(BYTE simbol, BYTE add, BYTE ves)
{
	BYTE byte = 0;
	simbol = (8 * simbol) | 0x40;
	
	lcd_com(simbol >> 4);
	lcd_com(simbol & 0x0F);
	
	for(BYTE i = 0; i <= 7; i++)
	{
		if(ves == 0) byte = pgm_read_byte(&(lcd_font_b[add][i]));
		if(ves == 1) byte = pgm_read_byte(&(lcd_font_s[add][i]));
		lcd_char_out(byte);
	}
	
	lcd_gotoxy(0,0);

}

//-------------------------------------------------------------------------
//	     
//	 :
//		BYTE *str -       
//		BYTE x -     X
//		BYTE y -     Y
//
//-------------------------------------------------------------------------

void lcd_rus_out(BYTE *str, BYTE x, BYTE y)
{
	//BYTE ttt[20];
	
	BYTE rus_str[20];	//    20
	BYTE i = 0;			//   CGRAM
	BYTE temp = 0;		//  
	BYTE j = 0;			//  
	BYTE ram = 0;

	while(*str != 0x00)
	{
		if(((*str != 0xD0) & (*str != 0xD1)))
		{ //       			
			if(*str > 0x7F)
				{ //   	
					if((*str > 0x8F) & (*str < 0xB0))
						{ //  
							
							temp = 0;
							while(temp < 10) // 
								{									
									if(pgm_read_byte(&lcd_rus_eng_b[temp][0]) == *str) //   
										{
											rus_str[j] = pgm_read_byte(&lcd_rus_eng_b[temp][1]);
										}
									temp++;
								} //---    
								
							temp = 0;	
							while(temp < 22)
								{
									if(pgm_read_byte(&lcd_rus_b[temp][0]) == *str) //   CGRAM
										{
											lcd_fsimbol(ram, pgm_read_byte(&lcd_rus_b[temp][1]), 0);
											rus_str[j] = ram;										
											ram++;	
										}
									temp++;
								} //---    CGRAM
						}
					else
						{ //  
							
							temp = 0;
							while(temp < 7) // 
								{
									if(pgm_read_byte(&lcd_rus_eng_s[temp][0]) == *str) //   
										{
											rus_str[j] = pgm_read_byte(&lcd_rus_eng_s[temp][1]);
										}
									temp++;
								} //---    
								
							temp = 0;
							while(temp < 25)
								{
									if(pgm_read_byte(&lcd_rus_s[temp][0]) == *str) //   CGRAM
										{
											lcd_fsimbol(ram, pgm_read_byte(&lcd_rus_s[temp][1]), 1);
											rus_str[j] = ram;
											ram++;
										}
									temp++;
								} //---    CGRAM	
						}
				}
			else
				{ //    
					rus_str[j] = *str;
				}
				
			j++; //   
			
			rus_str[j] = 0xFF; //  
		} //    D0 D1
		
		str++; //  
		
	} //   while()
	
	//   
	
	temp = 0;
	j = 0;
	
	lcd_gotoxy(x,y);
	
	while(rus_str[temp] != 0xFF)
	{
		lcd_char_out(rus_str[temp]);
		temp++;
	}
	
}

#endif

#endif /* LCD_H_ */ 