
//-------------------------------------------------<                axlib v1.1                 >----------------------------------------------------
//-------------------------------------------------<           USART       >----------------------------------------------------
//-------------------------------------------------<   2015 http://www.avrki.ru >----------------------------------------------------

#ifndef USART_H_
#define USART_H_




#if !defined(MAIN_INIT_H_)
#error "You must included (#include \"main_init.h\") befor use (#include <axlib/usart.h>)."
#endif

//-------------------------------------------------------------------------
//						  
//-------------------------------------------------------------------------

#define RXCIE_ON	(UCSRB |= (1 << RXCIE))
#define RXCIE_OFF	(UCSRB &= ~(1 << RXCIE))
#define TXCIE_ON	(UCSRB |= (1 << TXCIE))
#define TXCIE_OFF	(UCSRB |= ~(1 << TXCIE))

//-------------------------------------------------------------------------
//							 
//-------------------------------------------------------------------------
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <axlib/type_var.h>
#include <axlib/cicle_buffer.h>

//-------------------------------------------------------------------------
//							 
//-------------------------------------------------------------------------

//------------------------------------------------------------------------- 
//	  USART.
//
//	 :
//
//		USART_WORD baudrate -       .
//-------------------------------------------------------------------------

void usart_init(DWORD baudrate)
{
#if defined(UCSR0A)
	UCSR0A = 0x00;
	UCSR0B |= (3 << TXEN0) | (1 << RXCIE0);
	UCSR0C = (3 << UCSZ00);
	
	UBRR0H = ((F_CPU/16/baudrate-1) >> 8);
	UBRR0L = (F_CPU/16/baudrate-1);
#else
	UCSRA = 0x00;
	UCSRB |= (3 << TXEN) | (1 << RXCIE);
	UCSRC = (1 << URSEL) | (3 << UCSZ0);
	
	UBRRH = ((F_CPU/16/baudrate-1) >> 8);
	UBRRL = (F_CPU/16/baudrate-1);
#endif
	
	
	sei();
}

//-------------------------------------------------------------------------
//	    USART.
//	 :
//
//		BYTE data -   ,     .
//						       USART.
//-------------------------------------------------------------------------

void usart_char_out(BYTE data)
{
#if defined(UCSR0A)
	while (!( UCSR0A & (1 << UDRE0)));
	UDR0 = data;
#else
	while (!( UCSRA & (1 << UDRE)));
	UDR = data;
#endif
}

//-------------------------------------------------------------------------
//	    USART.
//
//	  :
//
//		BYTE -     USART.
//					      USART.
//-------------------------------------------------------------------------

BYTE usart_char_in(void)
{
#if defined(UCSR0A)
	while (!(UCSR0A & (1 << RXC0)));
	return UDR0;
#else
	while (!(UCSRA & (1 << RXC)));
	return UDR;
#endif
}

//-------------------------------------------------------------------------
//	    USART.
//
//	 :
//
//		BYTE *str -        
//						  ,     .
//-------------------------------------------------------------------------

void usart_str_out(BYTE *str, BYTE count)
{
	BYTE i = 0;
	while(i < count)
	{
		usart_char_out(*str);
		str++;
		i++;
	}
}

//-------------------------------------------------------------------------
//	    USART  RS-485 MH-BUS.
//
//	 :
//
//		BYTE *str -       
//						  ,     .
//						  
//-------------------------------------------------------------------------

void usart_mhbus_out(BYTE *str)
{
	BYTE i = 0;
	while(i < 11)
	{
		usart_char_out(*str);
		str++;
		i++;
	}
}

//-------------------------------------------------------------------------
//	       USART
//
//	  :
//
//			BYTE *str -      
//							      
//							   .    
//							   .    
//							       .
//-------------------------------------------------------------------------

BYTE usart_str_in(BYTE *str, BYTE count)
{
	BYTE out = 0;
	BYTE data = 0;
	BYTE timeout = 0;
	
	while(count > data)
	{
		data = GetData();
		
		//        
		//       0
		if(timeout >= 500)
		{
			ClearBuffer();
			return out;
		}
		
		timeout++;
		_delay_ms(1);
	}
	
	out = OutBufferStr(str, count);
	return out;
}

//-------------------------------------------------------------------------
//	    USART.
//
//	 :
//
//		BYTE *str -       
//						  ,     .
//						      
//						    \0.     
//						      \r\n. 
//-------------------------------------------------------------------------

void usart_str_rn(BYTE *str)
{
	BYTE i=0;
	while(str[i])
	{
		usart_char_out(str[i]);
		i++;
	}
	usart_char_out('\r');
	usart_char_out('\n');
	
}

//-------------------------------------------------------------------------
//	        USART
//
//	 :
//
//			BYTE -    
//-------------------------------------------------------------------------

BYTE usart_data(void)
{
	return GetData();
}

//-------------------------------------------------------------------------



#endif /* USART_H_ */